@@ -229,7 +229,7 @@ SQLRETURN SQLDisconnect( SQLHDBC connection_handle )
                 ERROR_25000, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
     else if ( connection -> state == STATE_C2 )
     {
@@ -243,7 +243,7 @@ SQLRETURN SQLDisconnect( SQLHDBC connection_handle )
                 ERROR_08003, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     /*
@@ -262,7 +262,7 @@ SQLRETURN SQLDisconnect( SQLHDBC connection_handle )
                 ERROR_HY010, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     if( __check_stmt_from_dbc( connection, STATE_S13 )) {
@@ -277,14 +277,15 @@ SQLRETURN SQLDisconnect( SQLHDBC connection_handle )
                 ERROR_HY010, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     /*
      * is it a pooled connection, or can it go back 
      */
 
-    if ( connection -> pooled_connection )
+    if ( connection -> pooled_connection ||
+         pooling_enabled && connection -> pooling_timeout > 0 )
     {
         __clean_stmt_from_dbc( connection );
         __clean_desc_from_dbc( connection );
@@ -304,29 +305,7 @@ SQLRETURN SQLDisconnect( SQLHDBC connection_handle )
                     connection -> msg );
         }
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_SUCCESS );
-    }
-    else if ( pooling_enabled && connection -> pooling_timeout > 0 ) 
-    {
-        __clean_stmt_from_dbc( connection );
-        __clean_desc_from_dbc( connection );
-
-        return_to_pool( connection );
-
-        if ( log_info.log_flag )
-        {
-            sprintf( connection -> msg, 
-                    "\n\t\tExit:[%s]",
-                        __get_return_status( SQL_SUCCESS, s1 ));
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    connection -> msg );
-        }
-
-        return function_return( SQL_HANDLE_DBC, connection, SQL_SUCCESS );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_SUCCESS );
     }
 
     /*
@@ -345,7 +324,7 @@ SQLRETURN SQLDisconnect( SQLHDBC connection_handle )
                 ERROR_IM001, NULL,
                 connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_DBC, connection, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_DBC, connection, SQL_ERROR );
     }
 
     ret = SQLDISCONNECT( connection,
@@ -359,7 +338,7 @@ SQLRETURN SQLDisconnect( SQLHDBC connection_handle )
 
         if ( ret == SQL_SUCCESS_WITH_INFO )
 	    {
-	        function_return_ex( IGNORE_THREAD, connection, ret, TRUE );
+	        function_return_ex( IGNORE_THREAD, connection, ret, TRUE, DEFER_R0 );
         }
         
         /*
@@ -382,5 +361,5 @@ SQLRETURN SQLDisconnect( SQLHDBC connection_handle )
                 connection -> msg );
     }
 
-    return function_return( SQL_HANDLE_DBC, connection, ret );
+    return function_return( SQL_HANDLE_DBC, connection, ret, DEFER_R0 );
 }
