@@ -230,7 +230,7 @@ SQLRETURN SQLExecute( SQLHSTMT statement_handle )
                     statement -> connection -> environment -> requested_version );
         }
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
     else if ( statement -> state == STATE_S1 ||
             statement -> state == STATE_S8 ||
@@ -250,7 +250,7 @@ SQLRETURN SQLExecute( SQLHSTMT statement_handle )
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( statement -> state == STATE_S11 ||
@@ -268,7 +268,7 @@ SQLRETURN SQLExecute( SQLHSTMT statement_handle )
                     ERROR_HY010, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
     }
 
@@ -284,7 +284,7 @@ SQLRETURN SQLExecute( SQLHSTMT statement_handle )
                 ERROR_IM001, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     ret = SQLEXECUTE( statement -> connection ,
@@ -301,7 +301,7 @@ SQLRETURN SQLExecute( SQLHSTMT statement_handle )
 
         if ( ret == SQL_SUCCESS_WITH_INFO )
         {
-            function_return_ex( IGNORE_THREAD, statement, ret, TRUE );
+            function_return_ex( IGNORE_THREAD, statement, ret, TRUE, DEFER_R1 );
         }
 
         local_ret = SQLNUMRESULTCOLS( statement -> connection,
@@ -360,5 +360,5 @@ SQLRETURN SQLExecute( SQLHSTMT statement_handle )
                 statement -> msg );
     }
 
-    return function_return( SQL_HANDLE_STMT, statement, ret );
+    return function_return( SQL_HANDLE_STMT, statement, ret, DEFER_R1 );
 }
