@@ -182,20 +182,26 @@
 
 static char const rcsid[]= "$RCSfile: SQLFreeHandle.c,v $ $Revision: 1.12 $";
 
+extern int pooling_enabled;
+
 SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
         SQLHANDLE handle )
 {
     switch( handle_type )
     {
       case SQL_HANDLE_ENV:
+      case SQL_HANDLE_SENV:
         {
             DMHENV environment = (DMHENV)handle;
 
             /*
-             * check environment
+             * check environment, the mark_released addition is to catch what seems to be a 
+             * race error in SQLAPI where it uses a env handle in one thread while its being released
+             * in another. releasing the handle at the end of this function is not fast enough for 
+             * the normal validation process to catch it.
              */
 
-            if ( !__validate_env( environment ))
+            if ( !__validate_env_mark_released( environment ))
             {
                 dm_log_write( __FILE__, 
                         __LINE__, 
@@ -224,6 +230,27 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
 
             thread_protect( SQL_HANDLE_ENV, environment );
 
+#ifdef WITH_SHARDENV
+            if ( pooling_enabled == 0 ) {
+                /*
+                 * check states
+                 */
+                if ( environment -> state != STATE_E1 )
+                {
+                    dm_log_write( __FILE__,
+                            __LINE__,
+                            LOG_INFO,
+                            LOG_INFO,
+                            "Error: HY010" );
+    
+                    __post_internal_error( &environment -> error,
+                            ERROR_HY010, NULL,
+                            environment -> requested_version );
+    
+                    return function_return_nodrv( SQL_HANDLE_ENV, environment, SQL_ERROR );
+                }
+            }
+#else
             /*
              * check states
              */
@@ -239,12 +266,25 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
                         ERROR_HY010, NULL,
                         environment -> requested_version );
 
-                return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_ENV, environment, SQL_ERROR );
             }
+#endif
 
             thread_release( SQL_HANDLE_ENV, environment );
 
+#ifdef WITH_SHARDENV
+            if ( pooling_enabled == 0 ) {
+                /*
+                 * release any pooled connections that are using this environment
+                 */
+                __strip_from_pool( environment );
+            }
+#else
+            __strip_from_pool( environment );
+#endif
+
             __release_env( environment );
+
             return SQL_SUCCESS;
         }
         break;
@@ -304,7 +344,7 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
                         ERROR_HY010, NULL,
                         connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_ENV, environment, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_ENV, environment, SQL_ERROR );
             }
 
             environment -> connection_count --;
@@ -408,7 +448,7 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
                           ERROR_HY010, NULL,
                           statement -> connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
             }
 
             if ( !CHECK_SQLFREEHANDLE( statement -> connection ))
@@ -425,7 +465,7 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
                             ERROR_IM001, NULL,
                             statement -> connection -> environment -> requested_version );
 
-                    return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+                    return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
                 }
                 else
                 {
@@ -443,16 +483,6 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
 
             if ( SQL_SUCCEEDED( ret ))
             {
-                /*
-                 * break any association
-                 */
-
-                if ( statement -> ard ) {
-                    statement -> ard -> associated_with = NULL;
-                }
-                if ( statement -> apd ) {
-                    statement -> apd -> associated_with = NULL;
-                }
                 /*
                  * release the implicit descriptors, 
 				 * this matches the tests in SQLAllocHandle
@@ -497,7 +527,7 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
                         connection -> msg );
             }
 
-            return function_return( IGNORE_THREAD, connection, ret ); 
+            return function_return( IGNORE_THREAD, connection, ret, DEFER_R0 );
         }
         break;
 
@@ -545,7 +575,7 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
 						ERROR_HY017, NULL,
 						connection -> environment -> requested_version );
 		
-				return function_return( IGNORE_THREAD, descriptor, SQL_ERROR );
+				return function_return_nodrv( IGNORE_THREAD, descriptor, SQL_ERROR );
 			}
 		
             thread_protect( SQL_HANDLE_DESC, descriptor );
@@ -562,7 +592,7 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
                         ERROR_IM001, NULL,
                         connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_DESC, descriptor, SQL_ERROR );
+                return function_return_nodrv( SQL_HANDLE_DESC, descriptor, SQL_ERROR );
             }
             else
             {
@@ -591,7 +621,7 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
                         ERROR_HY010, NULL,
                         descriptor -> connection -> environment -> requested_version );
 
-                return function_return( SQL_HANDLE_DESC, descriptor, SQL_ERROR );
+                return function_return( SQL_HANDLE_DESC, descriptor, SQL_ERROR, DEFER_R0 );
             }
 
             thread_release( SQL_HANDLE_DESC, descriptor );
@@ -614,7 +644,7 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
                                UODBC_STATS_TYPE_HDESC, (void *)-1);
 #endif
 
-            return function_return( IGNORE_THREAD, connection, SQL_SUCCESS ); 
+            return function_return( IGNORE_THREAD, connection, SQL_SUCCESS, DEFER_R0 );
         }
         break;
 
@@ -622,7 +652,7 @@ SQLRETURN __SQLFreeHandle( SQLSMALLINT handle_type,
         /*
          * there is nothing to report a error on
          */
-        return SQL_ERROR;
+        return SQL_INVALID_HANDLE;
     }
 }
 
