@@ -101,6 +101,89 @@ static char const rcsid[]= "$RCSfile: SQLGetDiagFieldW.c,v $";
 HY099,HY100,HY101,HY105,HY107,HY109,HY110,HY111,HYT00,HYT01,IM001,IM002,IM003,\
 IM004,IM005,IM006,IM007,IM008,IM010,IM011,IM012"
 
+SQLRETURN extract_parent_handle_field( SQLHANDLE handle,
+        int         handle_type,
+        SQLSMALLINT rec_number,
+        SQLSMALLINT diag_identifier,
+        SQLPOINTER  diag_info_ptr,
+        SQLSMALLINT buffer_length,
+        SQLSMALLINT *string_length_ptr )
+{
+    dm_log_write( __FILE__,
+            __LINE__,
+            LOG_INFO,
+            LOG_INFO,
+            "Error: SQL_INVALID_HANDLE" );
+
+    if ( handle_type != SQL_HANDLE_ENV )
+    {
+#ifdef WITH_HANDLE_REDIRECT
+        {
+            DMHDBC *parent_handle_dbc;
+            DMHSTMT *parent_handle_stmt;
+            DMHDESC *parent_handle_desc;
+
+            switch ( handle_type )
+            {
+                case SQL_HANDLE_DBC:
+                    {
+                        parent_handle_dbc = find_parent_handle( handle, handle_type );
+                    }
+                    break;
+
+                case SQL_HANDLE_STMT:
+                    {
+                        parent_handle_stmt = find_parent_handle( handle, handle_type );
+                        parent_handle_dbc = parent_handle_stmt->connection;
+                    }
+                    break;
+
+                case SQL_HANDLE_DESC:
+                    {
+                        parent_handle_desc = find_parent_handle( handle, handle_type );
+                        parent_handle_dbc = parent_handle_desc->connection;
+                    }
+                    break;
+
+                default:
+                    {
+                        return SQL_INVALID_HANDLE;
+                    }
+            }
+
+            if ( parent_handle_dbc )
+            {
+                dm_log_write( __FILE__,
+                        __LINE__,
+                        LOG_INFO,
+                        LOG_INFO,
+                        "Info: found parent handle" );
+
+                if ( CHECK_SQLGETDIAGFIELDW( parent_handle_dbc ))
+                {
+                    dm_log_write( __FILE__,
+                            __LINE__,
+                            LOG_INFO,
+                            LOG_INFO,
+                            "Info: calling redirected driver function" );
+
+                    return SQLGETDIAGFIELDW( parent_handle_dbc,
+                            handle_type,
+                            handle,
+                            rec_number,
+                            diag_identifier,
+                            diag_info_ptr,
+                            buffer_length,
+                            string_length_ptr );
+                }
+            }
+        }
+#endif
+    }
+
+    return SQL_INVALID_HANDLE;
+}
+
 /*
  * is it a diag identifier that we have to convert from unicode to ansi
  */
@@ -110,6 +193,7 @@ static int is_char_diag( int diag_identifier )
     switch( diag_identifier ) {
         case SQL_DIAG_CLASS_ORIGIN:
         case SQL_DIAG_CONNECTION_NAME:
+        case SQL_DIAG_DYNAMIC_FUNCTION:
         case SQL_DIAG_MESSAGE_TEXT:
         case SQL_DIAG_SERVER_NAME:
         case SQL_DIAG_SQLSTATE:
@@ -130,6 +214,11 @@ static SQLRETURN extract_sql_error_field_w( EHEAD *head,
 {
     ERROR *ptr;
 
+    if ( is_char_diag( diag_identifier ) && buffer_length < 0 )
+    {
+        return SQL_ERROR;
+    }
+
     /*
      * check the header fields first
      */
@@ -139,10 +228,10 @@ static SQLRETURN extract_sql_error_field_w( EHEAD *head,
       case SQL_DIAG_CURSOR_ROW_COUNT:
       case SQL_DIAG_ROW_COUNT:
         {
-            SQLINTEGER val;
+            SQLLEN val;
             SQLRETURN ret;
 
-            if ( head -> handle_type != SQL_HANDLE_STMT )
+            if ( rec_number > 0 || head -> handle_type != SQL_HANDLE_STMT )
             {
                 return SQL_ERROR;
             }
@@ -220,10 +309,22 @@ static SQLRETURN extract_sql_error_field_w( EHEAD *head,
         {
             SQLRETURN ret;
 
-            if ( head -> handle_type != SQL_HANDLE_STMT )
+            if ( rec_number > 0 )
             {
                 return SQL_ERROR;
             }
+            else if ( head -> handle_type != SQL_HANDLE_STMT )
+            {
+                if ( diag_info_ptr )
+                {
+                    *((SQLWCHAR*)diag_info_ptr) = 0;
+                }
+                if ( string_length_ptr )
+                {
+                    *string_length_ptr = 0;
+                }
+                return SQL_SUCCESS;
+            }
             else if ( head -> header_set )
             {
                 if ( SQL_SUCCEEDED( head -> diag_dynamic_function_ret ) && diag_info_ptr )
@@ -297,10 +398,15 @@ static SQLRETURN extract_sql_error_field_w( EHEAD *head,
             SQLINTEGER val;
             SQLRETURN ret;
 
-            if ( head -> handle_type != SQL_HANDLE_STMT )
+            if ( rec_number > 0  )
             {
                 return SQL_ERROR;
             }
+            else if ( head -> handle_type != SQL_HANDLE_STMT )
+            {
+                *((SQLINTEGER*)diag_info_ptr) = 0;
+                return SQL_SUCCESS;
+            }
             else if ( head -> header_set )
             {
                 if ( SQL_SUCCEEDED( head -> diag_dynamic_function_code_ret ) && diag_info_ptr )
@@ -353,6 +459,10 @@ static SQLRETURN extract_sql_error_field_w( EHEAD *head,
         {
             SQLINTEGER val;
             
+            if ( rec_number > 0 )
+            {
+                return SQL_ERROR;
+            }
             val = head -> sql_diag_head.internal_count + 
                 head -> sql_diag_head.error_count;
 
@@ -378,7 +488,9 @@ static SQLRETURN extract_sql_error_field_w( EHEAD *head,
      * else check the records
      */
 
-    if ( rec_number < 1 )
+    if ( rec_number < 1 ||
+        (( diag_identifier == SQL_DIAG_COLUMN_NUMBER ||
+        diag_identifier == SQL_DIAG_ROW_NUMBER ) && head -> handle_type != SQL_HANDLE_STMT ))
     {
         return SQL_ERROR;
     }
@@ -679,365 +791,145 @@ SQLRETURN SQLGetDiagFieldW( SQLSMALLINT handle_type,
     SQLRETURN ret;
     SQLCHAR s1[ 100 + LOG_MESSAGE_LEN ];
 
-    if ( handle_type == SQL_HANDLE_ENV )
+    DMHENV environment = ( DMHENV ) handle;
+    DMHDBC connection = NULL;
+    DMHSTMT statement = NULL;
+    DMHDESC descriptor = NULL;
+
+    EHEAD *herror;
+    char *handle_msg;
+    const char *handle_type_ptr;
+
+    switch ( handle_type )
     {
-        DMHENV environment = ( DMHENV ) handle;
+        case SQL_HANDLE_ENV:
+            {
+                if ( !__validate_env( environment ))
+                {
+                    return extract_parent_handle_field( environment, handle_type, rec_number, diag_identifier, diag_info_ptr, buffer_length, string_length_ptr );
+                }
 
-        if ( !__validate_env( environment ))
-        {
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    "Error: SQL_INVALID_HANDLE" );
+                herror = &environment->error;
+                handle_msg = environment->msg;
+                handle_type_ptr = "Environment";
+            }
+            break;
 
-            return SQL_INVALID_HANDLE;
-        }
+        case SQL_HANDLE_DBC:
+            {
+                connection = ( DMHDBC ) handle;
 
-        thread_protect( SQL_HANDLE_ENV, environment );
+                if ( !__validate_dbc( connection ))
+                {
+                    return extract_parent_handle_field( connection, handle_type, rec_number, diag_identifier, diag_info_ptr, buffer_length, string_length_ptr );
+                }
 
-        if ( log_info.log_flag )
-        {
-            sprintf( environment -> msg, 
-                "\n\t\tEntry:\
-\n\t\t\tEnvironment = %p\
+                herror = &connection->error;
+                handle_msg = connection->msg;
+                handle_type_ptr = "Connection";
+            }
+            break;
+
+        case SQL_HANDLE_STMT:
+            {
+                statement = ( DMHSTMT ) handle;
+
+                if ( !__validate_stmt( statement ))
+                {
+                    return extract_parent_handle_field( statement, handle_type, rec_number, diag_identifier, diag_info_ptr, buffer_length, string_length_ptr );
+                }
+
+                connection = statement->connection;
+                herror = &statement->error;
+                handle_msg = statement->msg;
+                handle_type_ptr = "Statement";
+            }
+            break;
+
+        case SQL_HANDLE_DESC:
+            {
+                descriptor = ( DMHDESC ) handle;
+
+                if ( !__validate_desc( descriptor ))
+                {
+                    return extract_parent_handle_field( descriptor, handle_type, rec_number, diag_identifier, diag_info_ptr, buffer_length, string_length_ptr );
+                }
+
+                connection = descriptor->connection;
+                herror = &descriptor->error;
+                handle_msg = descriptor->msg;
+                handle_type_ptr = "Descriptor";
+            }
+            break;
+
+        default:
+            {
+                return SQL_NO_DATA;
+            }
+    }
+
+    thread_protect( handle_type, handle );
+
+    if ( log_info.log_flag )
+    {
+        sprintf( handle_msg,
+            "\n\t\tEntry:\
+\n\t\t\t%s = %p\
 \n\t\t\tRec Number = %d\
 \n\t\t\tDiag Ident = %d\
 \n\t\t\tDiag Info Ptr = %p\
 \n\t\t\tBuffer Length = %d\
 \n\t\t\tString Len Ptr = %p",
-                    environment,
-                    rec_number,
-                    diag_identifier,
-                    diag_info_ptr,
-                    buffer_length,
-                    string_length_ptr );
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    environment -> msg );
-        }
-
-        ret = extract_sql_error_field_w( &environment -> error,
+                handle_type_ptr,
+                handle,
                 rec_number,
                 diag_identifier,
                 diag_info_ptr,
                 buffer_length,
                 string_length_ptr );
 
-        if ( log_info.log_flag )
-        {
-            sprintf( environment -> msg, 
-                    "\n\t\tExit:[%s]",
-                        __get_return_status( ret, s1 ));
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    environment -> msg );
-        }
-
-        thread_release( SQL_HANDLE_ENV, environment );
-
-        return ret;
+        dm_log_write( __FILE__,
+                __LINE__,
+                LOG_INFO,
+                LOG_INFO,
+                handle_msg );
     }
-    else if ( handle_type == SQL_HANDLE_DBC )
+
+    /*
+     * Do diag extraction here if defer flag is set.
+     * Clean the flag after extraction.
+     */
+    if ( connection && herror->defer_extract )
     {
-        DMHDBC connection = ( DMHDBC ) handle;
+        extract_error_from_driver( herror, connection, herror->ret_code_deferred, 0 );
 
-        if ( !__validate_dbc( connection ))
-        {
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    "Error: SQL_INVALID_HANDLE" );
+        herror->defer_extract = 0;
+        herror->ret_code_deferred = 0;
+    }
 
-#ifdef WITH_HANDLE_REDIRECT
-		{
-			DMHDBC parent_connection;
-
-			parent_connection = find_parent_handle( connection, SQL_HANDLE_DBC );
-
-			if ( parent_connection ) {
-        		dm_log_write( __FILE__, 
-                	__LINE__, 
-                    	LOG_INFO, 
-                    	LOG_INFO, 
-                    	"Info: found parent handle" );
-
-				if ( CHECK_SQLGETDIAGFIELDW( parent_connection ))
-				{
-        			dm_log_write( __FILE__, 
-                		__LINE__, 
-                   		 	LOG_INFO, 
-                   		 	LOG_INFO, 
-                   		 	"Info: calling redirected driver function" );
-
-					return SQLGETDIAGFIELDW( parent_connection, 
-							handle_type,
-							connection, 
-        					rec_number,
-        					diag_identifier,
-        					diag_info_ptr,
-        					buffer_length,
-        					string_length_ptr );
-				}
-			}
-		}
-#endif
-            return SQL_INVALID_HANDLE;
-        }
-
-        thread_protect( SQL_HANDLE_DBC, connection );
-
-        if ( log_info.log_flag )
-        {
-            sprintf( connection -> msg, 
-                "\n\t\tEntry:\
-\n\t\t\tConnection = %p\
-\n\t\t\tRec Number = %d\
-\n\t\t\tDiag Ident = %d\
-\n\t\t\tDiag Info Ptr = %p\
-\n\t\t\tBuffer Length = %d\
-\n\t\t\tString Len Ptr = %p",
-                    connection,
-                    rec_number,
-                    diag_identifier,
-                    diag_info_ptr,
-                    buffer_length,
-                    string_length_ptr );
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    connection -> msg );
-        }
-
-        ret = extract_sql_error_field_w( &connection -> error,
+    ret = extract_sql_error_field_w( herror,
                 rec_number,
                 diag_identifier,
                 diag_info_ptr,
                 buffer_length,
                 string_length_ptr );
 
-        if ( log_info.log_flag )
-        {
-            sprintf( connection -> msg, 
-                    "\n\t\tExit:[%s]",
-                        __get_return_status( ret, s1 ));
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    connection -> msg );
-        }
-
-        thread_release( SQL_HANDLE_DBC, connection );
-
-        return ret;
-    }
-    else if ( handle_type == SQL_HANDLE_STMT )
+    if ( log_info.log_flag )
     {
-        DMHSTMT statement = ( DMHSTMT ) handle;
+        sprintf( handle_msg,
+            "\n\t\tExit:[%s]",
+            __get_return_status( ret, s1 ));
 
-        if ( !__validate_stmt( statement ))
-        {
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    "Error: SQL_INVALID_HANDLE" );
-
-#ifdef WITH_HANDLE_REDIRECT
-		{
-			DMHSTMT parent_statement;
-
-			parent_statement = find_parent_handle( statement, SQL_HANDLE_STMT );
-
-			if ( parent_statement ) {
-        		dm_log_write( __FILE__, 
-                	__LINE__, 
-                    	LOG_INFO, 
-                    	LOG_INFO, 
-                    	"Info: found parent handle" );
-
-				if ( CHECK_SQLGETDIAGFIELDW( parent_statement -> connection ))
-				{
-        			dm_log_write( __FILE__, 
-                		__LINE__, 
-                   		 	LOG_INFO, 
-                   		 	LOG_INFO, 
-                   		 	"Info: calling redirected driver function" );
-
-					return SQLGETDIAGFIELDW( parent_statement -> connection, 
-							handle_type,
-							statement, 
-        					rec_number,
-        					diag_identifier,
-        					diag_info_ptr,
-        					buffer_length,
-        					string_length_ptr );
-				}
-			}
-		}
-#endif
-            return SQL_INVALID_HANDLE;
-        }
-
-        thread_protect( SQL_HANDLE_STMT, statement );
-
-        if ( log_info.log_flag )
-        {
-            sprintf( statement -> msg, 
-                "\n\t\tEntry:\
-\n\t\t\tStatement = %p\
-\n\t\t\tRec Number = %d\
-\n\t\t\tDiag Ident = %d\
-\n\t\t\tDiag Info Ptr = %p\
-\n\t\t\tBuffer Length = %d\
-\n\t\t\tString Len Ptr = %p",
-                    statement,
-                    rec_number,
-                    diag_identifier,
-                    diag_info_ptr,
-                    buffer_length,
-                    string_length_ptr );
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    statement -> msg );
-        }
-
-        ret = extract_sql_error_field_w( &statement -> error,
-                rec_number,
-                diag_identifier,
-                diag_info_ptr,
-                buffer_length,
-                string_length_ptr );
-
-        if ( log_info.log_flag )
-        {
-            sprintf( statement -> msg, 
-                    "\n\t\tExit:[%s]",
-                        __get_return_status( ret, s1 ));
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    statement -> msg );
-        }
-
-        thread_release( SQL_HANDLE_STMT, statement );
-
-        return ret;
+        dm_log_write( __FILE__,
+                __LINE__,
+                LOG_INFO,
+                LOG_INFO,
+                handle_msg );
     }
-    else if ( handle_type == SQL_HANDLE_DESC )
-    {
-        DMHDESC descriptor = ( DMHDESC ) handle;
 
-        if ( !__validate_desc( descriptor ))
-        {
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    "Error: SQL_INVALID_HANDLE" );
+    thread_release( handle_type, handle );
 
-#ifdef WITH_HANDLE_REDIRECT
-		{
-			DMHDESC parent_desc;
+    return ret;
 
-			parent_desc = find_parent_handle( descriptor, SQL_HANDLE_DESC );
-
-			if ( parent_desc ) {
-        		dm_log_write( __FILE__, 
-                	__LINE__, 
-                    	LOG_INFO, 
-                    	LOG_INFO, 
-                    	"Info: found parent handle" );
-
-				if ( CHECK_SQLGETDIAGFIELDW( parent_desc -> connection ))
-				{
-        			dm_log_write( __FILE__, 
-                		__LINE__, 
-                   		 	LOG_INFO, 
-                   		 	LOG_INFO, 
-                   		 	"Info: calling redirected driver function" );
-
-					return SQLGETDIAGFIELDW( parent_desc -> connection, 
-							handle_type,
-							descriptor,
-        					rec_number,
-        					diag_identifier,
-        					diag_info_ptr,
-        					buffer_length,
-        					string_length_ptr );
-				}
-			}
-		}
-#endif
-            return SQL_INVALID_HANDLE;
-        }
-
-        thread_protect( SQL_HANDLE_DESC, descriptor );
-
-        if ( log_info.log_flag )
-        {
-            sprintf( descriptor -> msg, 
-                "\n\t\tEntry:\
-\n\t\t\tDescriptor = %p\
-\n\t\t\tRec Number = %d\
-\n\t\t\tDiag Ident = %d\
-\n\t\t\tDiag Info Ptr = %p\
-\n\t\t\tBuffer Length = %d\
-\n\t\t\tString Len Ptr = %p",
-                    descriptor,
-                    rec_number,
-                    diag_identifier,
-                    diag_info_ptr,
-                    buffer_length,
-                    string_length_ptr );
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    descriptor -> msg );
-        }
-
-        ret = extract_sql_error_field_w( &descriptor -> error,
-                rec_number,
-                diag_identifier,
-                diag_info_ptr,
-                buffer_length,
-                string_length_ptr );
-
-        if ( log_info.log_flag )
-        {
-            sprintf( descriptor -> msg, 
-                    "\n\t\tExit:[%s]",
-                        __get_return_status( ret, s1 ));
-
-            dm_log_write( __FILE__, 
-                    __LINE__, 
-                    LOG_INFO, 
-                    LOG_INFO, 
-                    descriptor -> msg );
-        }
-
-        thread_release( SQL_HANDLE_DESC, descriptor );
-
-        return ret;
-    }
-    return SQL_NO_DATA;
 }
 
