@@ -184,7 +184,7 @@ SQLRETURN SQLMoreResults( SQLHSTMT statement_handle )
                 LOG_INFO, 
                 statement -> msg );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_NO_DATA );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_NO_DATA );
     }
     else if ( statement -> state == STATE_S8 ||
             statement -> state == STATE_S9 ||
@@ -200,7 +200,7 @@ SQLRETURN SQLMoreResults( SQLHSTMT statement_handle )
                 ERROR_HY010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
     else if ( statement -> state == STATE_S11 ||
             statement -> state == STATE_S12 )
@@ -217,25 +217,9 @@ SQLRETURN SQLMoreResults( SQLHSTMT statement_handle )
                     ERROR_HY010, NULL,
                     statement -> connection -> environment -> requested_version );
 
-            return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+            return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
         }
     }
-    else if ( statement -> state == STATE_S13 ||
-            statement -> state == STATE_S14 ||
-            statement -> state == STATE_S15 )
-    {
-        dm_log_write( __FILE__, 
-                __LINE__, 
-                LOG_INFO, 
-                LOG_INFO, 
-                "Error: HY010" );
-
-        __post_internal_error( &statement -> error,
-                ERROR_HY010, NULL,
-                statement -> connection -> environment -> requested_version );
-
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
-    }
 
 #ifdef NR_PROBE
     if ( !CHECK_SQLMORERESULTS( statement -> connection ) ||
@@ -254,7 +238,7 @@ SQLRETURN SQLMoreResults( SQLHSTMT statement_handle )
                 ERROR_IM001, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     ret = SQLMORERESULTS( statement -> connection ,
@@ -269,7 +253,7 @@ SQLRETURN SQLMoreResults( SQLHSTMT statement_handle )
 
         if ( ret == SQL_SUCCESS_WITH_INFO )
         {
-            function_return_ex( IGNORE_THREAD, statement, ret, TRUE );
+            function_return_ex( IGNORE_THREAD, statement, ret, TRUE, DEFER_R3 );
         }
 
         SQLNUMRESULTCOLS( statement -> connection,
@@ -348,5 +332,5 @@ SQLRETURN SQLMoreResults( SQLHSTMT statement_handle )
                 statement -> msg );
     }
 
-    return function_return( SQL_HANDLE_STMT, statement, ret );
+    return function_return( SQL_HANDLE_STMT, statement, ret, DEFER_R3 );
 }
