@@ -185,7 +185,7 @@ SQLRETURN SQLParamOptions(
                 ERROR_S1107, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     /*
@@ -196,7 +196,10 @@ SQLRETURN SQLParamOptions(
             statement -> state == STATE_S9 ||
             statement -> state == STATE_S10 ||
             statement -> state == STATE_S11 ||
-            statement -> state == STATE_S12 )
+            statement -> state == STATE_S12 ||
+            statement -> state == STATE_S13 ||
+            statement -> state == STATE_S14 ||
+            statement -> state == STATE_S15 )
     {
         dm_log_write( __FILE__, 
                 __LINE__, 
@@ -208,7 +211,7 @@ SQLRETURN SQLParamOptions(
                 ERROR_S1010, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( CHECK_SQLPARAMOPTIONS( statement -> connection ))
@@ -223,7 +226,7 @@ SQLRETURN SQLParamOptions(
         ret = SQLSETSTMTATTR( statement -> connection,
                 statement -> driver_stmt,
                 SQL_ATTR_PARAMSET_SIZE, 
-                crow, 
+                (SQLPOINTER)(intptr_t) crow,
                 0 );
         if ( SQL_SUCCEEDED( ret ))
         {
@@ -239,7 +242,7 @@ SQLRETURN SQLParamOptions(
         ret = SQLSETSTMTATTRW( statement -> connection,
                 statement -> driver_stmt,
                 SQL_ATTR_PARAMSET_SIZE, 
-                crow, 
+                (SQLPOINTER)(intptr_t) crow,
                 0 );
         if ( SQL_SUCCEEDED( ret ))
         {
@@ -262,7 +265,7 @@ SQLRETURN SQLParamOptions(
                 ERROR_IM001, NULL,
                 statement -> connection -> environment -> requested_version );
 
-        return function_return( SQL_HANDLE_STMT, statement, SQL_ERROR );
+        return function_return_nodrv( SQL_HANDLE_STMT, statement, SQL_ERROR );
     }
 
     if ( log_info.log_flag )
@@ -278,5 +281,5 @@ SQLRETURN SQLParamOptions(
                 statement -> msg );
     }
 
-    return function_return( SQL_HANDLE_STMT, statement, ret );
+    return function_return( SQL_HANDLE_STMT, statement, ret, DEFER_R3 );
 }
