@@ -1013,7 +1013,7 @@ static void __set_attribute( void *handle, int type, struct attr_set *as )
                     ret = SQLSETENVATTR( connection,
                             connection -> driver_dbc,
                             as -> attribute,
-                            as -> int_value,
+                            (SQLPOINTER)(intptr_t) as -> int_value,
                             0 );
                 }
                 else
@@ -1047,7 +1047,7 @@ static void __set_attribute( void *handle, int type, struct attr_set *as )
                     ret = SQLSETCONNECTATTR( connection,
                             connection -> driver_dbc,
                             as -> attribute,
-                            as -> int_value,
+                            (SQLPOINTER)(intptr_t) as -> int_value,
                             0 );
                 }
                 else
@@ -1073,7 +1073,7 @@ static void __set_attribute( void *handle, int type, struct attr_set *as )
                     ret = SQLSETCONNECTOPTION( connection,
                             connection -> driver_dbc,
                             as -> attribute,
-                            as -> value );
+                            (SQLULEN) as -> value );
                 }
             }
         }
@@ -1093,7 +1093,7 @@ static void __set_attribute( void *handle, int type, struct attr_set *as )
                     ret = SQLSETCONNECTOPTION( connection,
                             connection -> driver_dbc,
                             as -> attribute,
-                            as -> value );
+                            (SQLULEN) as -> value );
                 }
             }
         }
@@ -1119,7 +1119,7 @@ static void __set_attribute( void *handle, int type, struct attr_set *as )
                     ret = SQLSETSTMTATTR( connection,
                             statement -> driver_stmt,
                             as -> attribute,
-                            as -> int_value,
+                            (SQLPOINTER)(intptr_t) as -> int_value,
                             0 );
                 }
                 else
@@ -1145,7 +1145,7 @@ static void __set_attribute( void *handle, int type, struct attr_set *as )
                     ret = SQLSETSTMTOPTION( connection,
                             statement -> driver_stmt,
                             as -> attribute,
-                            as -> value );
+                            (SQLULEN) as -> value );
                 }
             }
         }
@@ -1165,7 +1165,7 @@ static void __set_attribute( void *handle, int type, struct attr_set *as )
                     ret = SQLSETSTMTOPTION( connection,
                             statement -> driver_stmt,
                             as -> attribute,
-                            as -> value );
+                            (SQLULEN) as -> value );
                 }
             }
 
@@ -1182,7 +1182,7 @@ static void __set_attribute( void *handle, int type, struct attr_set *as )
                         ret = SQLSETSTMTATTR( connection,
                                 statement -> driver_stmt,
                                 as -> attribute,
-                                as -> int_value,
+                                (SQLPOINTER)(intptr_t) as -> int_value,
                                 0 );
                     }
                     else
@@ -1302,7 +1302,11 @@ void *__attr_override( void *handle, int type, int attribute, void *value, SQLIN
 
         if ( as -> is_int_type )
         {
+#ifdef HAVE_PTRDIFF_T
+            return (void*)(ptrdiff_t) as -> int_value;
+#else
             return (void*)(long) as -> int_value;
+#endif
         }
         else
         {
@@ -1364,7 +1368,11 @@ void *__attr_override_wide( void *handle, int type, int attribute, void *value,
 
         if ( as -> is_int_type )
         {
+#ifdef HAVE_PTRDIFF_T
+            return (void*)(ptrdiff_t) as -> int_value;
+#else
             return (void*)(long) as -> int_value;
+#endif
         }
         else
         {
@@ -1397,9 +1405,17 @@ void *__attr_override_wide( void *handle, int type, int attribute, void *value,
 
 int dm_check_connection_attrs( DMHDBC connection, SQLINTEGER attribute, SQLPOINTER value )
 {
+#ifdef HAVE_PTRDIFF_T
+    ptrdiff_t ival;
+#else
     SQLINTEGER ival;
+#endif
 
+#ifdef HAVE_PTRDIFF_T
+    ival = (ptrdiff_t) value;
+#else
     ival = (SQLINTEGER) value;
+#endif
 
     switch( attribute ) 
     {
@@ -1453,6 +1469,14 @@ int dm_check_connection_attrs( DMHDBC connection, SQLINTEGER attribute, SQLPOINT
             }
             break;
 
+        case SQL_ATTR_TXN_ISOLATION:
+            if ( ival != SQL_TXN_READ_UNCOMMITTED && ival != SQL_TXN_READ_COMMITTED
+                && ival != SQL_TXN_REPEATABLE_READ && ival != SQL_TXN_SERIALIZABLE )
+            {
+                return SQL_ERROR;
+            }
+            break;
+
         /*
          * include statement attributes as well
          */
@@ -1534,9 +1558,17 @@ int dm_check_connection_attrs( DMHDBC connection, SQLINTEGER attribute, SQLPOINT
 
 int dm_check_statement_attrs( DMHSTMT statement, SQLINTEGER attribute, SQLPOINTER value )
 {
+#ifdef HAVE_PTRDIFF_T
+    ptrdiff_t ival;
+#else
     SQLUINTEGER ival;
+#endif
 
+#ifdef HAVE_PTRDIFF_T
+    ival = (ptrdiff_t) value;
+#else
     ival = (SQLUINTEGER) value;
+#endif
 
     switch( attribute ) 
     {
@@ -1615,6 +1647,9 @@ int dm_check_statement_attrs( DMHSTMT statement, SQLINTEGER attribute, SQLPOINTE
             }
             break;
 
+        case SQL_ROWSET_SIZE:
+            return ival > 0 ? SQL_SUCCESS : SQL_ERROR;
+
         default:
             return SQL_SUCCESS;
     }
