@@ -18,6 +18,9 @@
  *
  */
 
+#include <config.h>
+
+#include <stdio.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -26,6 +29,7 @@
 #include "connection.h"
 #include "qresult.h"
 #include "pgtypes.h"
+#include "misc.h"
 
 #ifndef WIN32
 #ifndef HAVE_STRICMP
