@@ -3,7 +3,7 @@
  * unixODBC Cursor Library
  *
  * Created by Nick Gorham
- * (nick@easysoft.com).
+ * (nick@lurcher.org).
  *
  * copyright (c) 1999 Nick Gorham
  *
@@ -23,9 +23,12 @@
  *
  **********************************************************************
  *
- * $Id: SQLParamData.c,v 1.2 2004/03/15 09:23:59 lurcher Exp $
+ * $Id: SQLParamData.c,v 1.3 2009/02/18 17:59:18 lurcher Exp $
  *
  * $Log: SQLParamData.c,v $
+ * Revision 1.3  2009/02/18 17:59:18  lurcher
+ * Shift to using config.h, the compile lines were making it hard to spot warnings
+ *
  * Revision 1.2  2004/03/15 09:23:59  lurcher
  *
  * Add SQL_NULL_DESC
@@ -46,6 +49,7 @@
  *
  **********************************************************************/
 
+#include <config.h>
 #include "cursorlibrary.h"
 
 SQLRETURN CLParamData( SQLHSTMT statement_handle,
